﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Collections.Generic;

namespace FaceParty
{
    public partial class EditPage : PhoneApplicationPage
    {
        protected override void OnOrientationChanged(OrientationChangedEventArgs e)
        {
            this.ApplyStyles(e.Orientation);
            base.OnOrientationChanged(e);
        }

        private void ApplyStyles(PageOrientation pageOrientation)
        {
            string suffix = GetStyleSuffix(pageOrientation);

            ApplyStylesRec(this.LayoutRoot, suffix);
        }

        private void ApplyStylesRec(FrameworkElement element, string suffix)
        {
            if (element == null)
            {
                return;
            }

            if (element.Style != null)
            {
                string oriStyle = FindOriginalStyle(element.Name);

                if (!string.IsNullOrEmpty(oriStyle))
                {
                    string newStyle = oriStyle + suffix;

                    if (Resources.Contains(newStyle))
                    {
                        element.Style = (Style)Resources[newStyle];
                    }
                }
            }

            // recusively for panels;            
            if (element is Panel)
            {
                Panel container = element as Panel;

                foreach (var item in container.Children)
                {
                    ApplyStylesRec(item as FrameworkElement, suffix);
                }
            }
            else if (element is Border)
            {
                ApplyStylesRec((element as Border).Child as FrameworkElement, suffix);
            }
        }

        /// <summary>
        /// Find suffix of style name for specified page orientation;
        /// </summary>
        private string GetStyleSuffix(PageOrientation pageOrientation)
        {
            if (pageOrientation == PageOrientation.LandscapeLeft)
            {
                return "LL";
            }
            else if (pageOrientation == PageOrientation.LandscapeRight)
            {
                return "LR";
            }
            else
            {
                // default suffix;
                return null;
            }
        }

        static Dictionary<string, string> RegisteredStyles = new Dictionary<string, string>()
        {
            {"ButtonPanel","buttonPanelStyle"},
            {"ButtonStack","stackpanelStyle"},
            {"PhotoPanel","photoPanelStyle"},
            {"EditGrid","EditGridStyle"},
            {"imgGrid","photoGridStyle"},
            {"EditButton","EditButtonStyle"},
            {"resetButton","ResetButtonStyle"},
            {"saveButton","SaveButtonStyle"},
            {"shareButton","ShareButtonStyle"},
            {"mmsgbox","msgboxStyle"}
        };

        private string FindOriginalStyle(string elementName)
        {
            if (string.IsNullOrEmpty(elementName))
            {
                return null;
            }

            if (RegisteredStyles.ContainsKey(elementName))
            {
                return RegisteredStyles[elementName];
            }
            else
            {
                return null;
            }
        }
    }
}
