﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.IO;
using UIFramework.Framework;
using Microsoft.Phone.Tasks;
using System.Windows.Navigation;
using System.ComponentModel;
using Microsoft.Xna.Framework.Media;
using FaceParty.Common;

namespace FaceParty
{
    public partial class MainPage : PhoneApplicationPage
    {
        private Persistence persistence = (App.Current as App).Persistence;
        /// <summary>
        /// mark for whether it's back from edit page when face detect failed
        /// </summary>
        public static bool IsBackFromFaceDetectionFailed { get; set; }
        // Constructor
        public MainPage()
        {
            InitializeComponent();

            InitRecentList();

            IsBackFromFaceDetectionFailed = false;
        }

        #region Add New Effect

        private Stream chosenPhoto;
        private bool isChoosing;

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            if (isChoosing)
            {
                return;
            }

            isChoosing = true;

            PhotoChooserTask photoChoose = new PhotoChooserTask();
            photoChoose.ShowCamera = true;
            photoChoose.Completed += (s, res) =>
            {
                isChoosing = false;

                if (res.TaskResult == TaskResult.Cancel && res.Error is InvalidOperationException)
                {
                    MessageBox.Show(Localization.Strings.message_duplicate_add);
                    return;
                }

                if (res.TaskResult == TaskResult.OK)
                {
                    chosenPhoto = res.ChosenPhoto;

                    // Since PhotoChooserTask is not completed yet to front, NavigationService.Navigate is not allowed here.
                    // This is a workaround by one time navigation event subscribed, and deplayed the navigation to another page.
                    NavigationService.Navigated += new NavigatedEventHandler(NavigationService_Navigated);
                }
            };
            photoChoose.Show();
        }

        void NavigationService_Navigated(object sender, NavigationEventArgs e)
        {
            if (chosenPhoto != null)
            {
                persistence.PhotoManager.BeginEditNewPhoto(chosenPhoto);

                NavigationService.Navigated -= new NavigatedEventHandler(NavigationService_Navigated);
                NavigationService.Navigate(new Uri(ConstantUrls.EditPage, UriKind.Relative));
            }
        }
        #endregion

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Get a dictionary of query string keys and values.
            IDictionary<string, string> queryStrings = this.NavigationContext.QueryString;

            // Ensure that there is at least one key in the query string, and check whether the "token" key is present.
            bool isTokenExist = queryStrings.ContainsKey("token");

            //when face detection failed, the app should back to the add photo item in the mainPage panorama
            //to fix the item change bug for panorama, we must change the DefaultItem from AddPhotoItem to null,then change it back to the AddPhotoItem
            //the IsBackFromFaceDetectionFailed property is to make sure only when face detection failed occurs.
            if (IsBackFromFaceDetectionFailed)
            {
                IsBackFromFaceDetectionFailed = false;
                this.MainPanorama.DefaultItem = null;
                this.MainPanorama.DefaultItem = this.AddPhotoItem;
            }

            LoadRecentList();

            // Ensure that there is at least one key in the query string, and check whether the "token" key is present.
            if (isTokenExist && e.NavigationMode != NavigationMode.Back)
            {
                isChoosing = false;

                // Retrieve the picture from the media library using the token passed to the application.
                MediaLibrary library = new MediaLibrary();
                Picture picture = library.GetPictureFromToken(queryStrings["token"]);

                chosenPhoto = picture.GetImage();
                persistence.PhotoManager.BeginEditNewPhoto(chosenPhoto);
                NavigationService.Navigate(new Uri(ConstantUrls.EditPage, UriKind.Relative));
            }

            base.OnNavigatedTo(e);
        }

        void LoadRecentList()
        {
            RecentList.ItemSource = persistence.PhotoManager.Photos;

            // keep one binding event;
            persistence.PhotoManager.PropertyChanged -= new PropertyChangedEventHandler(PhotoManager_PropertyChanged);
            persistence.PhotoManager.PropertyChanged += new PropertyChangedEventHandler(PhotoManager_PropertyChanged);
        }

        void PhotoManager_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            LoadRecentList();
        }

        void InitRecentList()
        {
            RecentList.Selected += (sender, e) =>
            {
                persistence.PhotoManager.BeginEditPhoto(RecentList.SelectedPhoto);
                NavigationService.Navigate(new Uri(ConstantUrls.EditPage, UriKind.Relative));
            };

            RecentList.Deleted += (sender, e) =>
            {
                persistence.PhotoManager.DeletePhoto(RecentList.DeletePhoto);
            };
        }
    }
}